/***********************************************************
*                       pC/TFS 2.38b                       *
*                 a FREE - Tiny File System                *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                      TFS_Port_RAM.h                      *
*                  RAM test DECLARATIONS                   *
***********************************************************/
#ifndef _TFSPort_RAM_
 #define _TFSPort_RAM_

 #include "../pC_TFS.h"

/*
***********************************************************
*          TFS-MEMORY AREA (RAM / FRAM / MRAM / RRAM)
*
* - all processors:       a part of the int./ext. RAM
* - SAM7SExx_DevMaster:   FRAM FM22L16      55ns   512kB
* - Open417I-C:           sRAM IS62WV51216  55ns     1MB
*                         MRAM MR4A16       35ns     2MB
* - HY-STM32F4xxCore144: PSRAM EM7644SU16   70ns     8MB
*                         MRAM MR4A16       35ns     2MB
*
*                        PCRAM ?
*                         RRAM ?
***********************************************************
*/
#ifdef _OS_8051_
 #define TFS_START     0x018000    // incl. segm-info for Keil
 #define TFS_SIZE      0x002000    // size of TFS (8kB)
#else
 #ifdef _OS_186_
  #define TFS_START    0x800000    // for Am188ES
  #define TFS_SIZE     0x200000    // size of TFS (512kB)
 #else
  #ifdef _OS_Rabbit_
   #define TFS_START   0x008000    // for Rabbit
   #define TFS_SIZE    0x002000    // size of TFS (8kB)
  #else
   #ifdef _OS_AT91SAM7Sxx_
    #define TFS_START  0x202000    // for AT91SAM7Sxx
    #define TFS_SIZE   0x004000    // size of TFS (16kB)
   #else
    #ifdef _OS_SAMD21_
     #define TFS_START 0x202000    // for SAMD21
     #define TFS_SIZE  0x004000    // size of TFS (16kB)
    #else
     #ifdef _OS_STM32F4xx_
      #include "../../../BSP/Open417I/Open417_FSMC_xRAM.h"
      #define TFS_START FSMC_MRAM_START_ADDR  // for Open417 board with MRAM
      #define TFS_SIZE  FSMC_MRAM_SIZE        // size of TFS (2MB)
     #else
      #error "unknown RAM-port !"
     #endif
    #endif
   #endif
  #endif
 #endif
#endif

/*
***********************************************************
*                     TFS-PORT PROTOTYPES
***********************************************************
*/
U08 TFSPort_Init(void) KC51_REENTRANT;
U08 TFSPort_Write(U08 OS_HUGE *dest, OS_CONST U08 OS_HUGE *source, TFS_LONG length) KC51_REENTRANT;
U08 TFSPort_Fill(U08 OS_HUGE *dest, U08 value, TFS_LONG length) KC51_REENTRANT;

#if defined(_WIN32) || defined(WIN32) || defined(__LINUX__) || defined(__linux__) || defined(__CYGWIN__)
 #define TFS_INVALID     -355                 // Image-File is invalid
 S16 TFS_Load();
 S16 TFS_Flush();
#endif

/**********************************************************/
#endif
