/***********************************************************
*                       pC/TFS 2.38b                       *
*                 a FREE - Tiny File System                *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                         TFS_cfg.h                        *
*                   pC/TFS CONFIGURATION                   *
***********************************************************/

#ifndef _TFS_cfg_
 #define _TFS_cfg_


#include "../../pC_OS/inc/pC_OS.h"               // only for use of pC/OS
                                                 // or your have to include ..
                                                 // .. your own definition file (types)
/*
************************************************************
*                        TFS CONFIG
************************************************************
*/
#define TFSDBLLINKED        1                    // double linked system
#define TFSLONGAUTO         1                    // automatic TFS_LONG switching
#define TFS_Name_SIZE    15+1                    // characters + C-termination
#define TFS_HANDLES         8                    // max files opened by a user (task)
#define TFS_SubROOT         1                    // use the user Sub-ROOT feature
#define TFS_TempDIR         1                    // use the one-level temorary current-Dir feature

/*
************************************************************
*                    SUB-CONFIGURATION
************************************************************
*/
#define TFS_AddMin      50                       // minimum size of a DB (DataBlock of a file)

/*
************************************************************
*                       OS ADAPTION
************************************************************
*/
#ifdef _pC_OS_
 #define OS_USER_ID     OSTCBCur->OSTCBId        // get unique task-ID
 #define TFS_MAX_USER   3                        // use max-tasks for max-users (max OS_MAX_TASKS)
 #define TFS_MUTEX_TYPE OS_SEM                   // use a semaphore on pC/OS to test sheduling during task(s) in TFS
#else
 #define OS_USER_ID     0                        // without a OS (only user-0 possible)
 #define TFS_MAX_USER   1                        // only one user (user-0)
 #define TFS_MUTEX_TYPE U08                      // use a uchar without an OS
#endif

/*
************************************************************
*           CONSTANTS & TYPES (NEVER CHANGE THIS)
************************************************************
*/
//#if ((TFS_SIZE < 0x10000UL) && (TFSLONGAUTO == 1))
// typedef U16        TFS_LONG;                    // for all variables using length or part of length of TFS
//#else
 typedef U32           TFS_LONG;                 // for all variables using length or part of length of TFS
//#endif

typedef U08            TFS_HANDLE;               // FileHandle is an byte
typedef U08            TFS_ATTR;                 // Attribs in an byte
typedef OS_CONST char  TFS_PATHNAME;             // use "const char" for pathes & names

/*
************************************************************
*                    TFS hardware PORT                     *
************************************************************
*/
// #include "../inc/RAM/TFS_Port_RAM.h"                         // std port to RAM-area / FRAM / MRAM / Linux / Windows32
 #include "../inc/AT28/TFS_Port_Atmel_EE.h"                   // port to AT28Cxxx Atmel-EE
// #include "../inc/AT91SAM7SExxx/TFS_Port_AT91SAM7SExxx.h"     // port to AT91SAM7SExxx NOR bank-1
// #include "../inc/SAMD21/TFS_Port_SAMD21.h"                   // port to SAMD21 NOR (no extra bank !)

/************************* END ****************************/
#endif
