/***********************************************************
*                       pC/TFS 2.38b                       *
*                 a FREE - Tiny File System                *
*            by Frank Goetze  -  www.embedded-os.de        *
************************************************************
*                 TFS_Port_AT91SAM7SExxx.c                 *
*                  AT91SAM7SExxx NOR Port                  *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/
#include "../../inc/AT91SAM7SExxx/TFS_Port_AT91SAM7SExxx.h"

#include <AT91SAM7SE512.h>

#define MCU_MAIN_CLOCK  48000000

#define AT91C_MC_CORRECT_KEY   ((unsigned int) 0x5A << 24)   

/*
************************************************************
*                      HW-specifics
************************************************************
*/
#define TFSport_NOR_PAGESIZE    256

// this version works only in Bank-1 NOR-flash !!!

#define NOR_BANK1_START         0x00140000
#define NOR_BANK_SIZE           0x00040000

static U08 TFSport_erase_page(U08 OS_HUGE *page_addr)
{
    return TFS_NO_ERR;                                                                                           // automatic erase during program cycle
}

static U08 TFSport_write_page(U08 OS_HUGE *page_addr, OS_CONST U08 OS_HUGE* src)
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U32 i, page = ((U32)page_addr - TFS_START) / TFSport_NOR_PAGESIZE;;

    OS_ENTER_CRITICAL();
    AT91C_BASE_MC->MC1_FCR = AT91C_MC_CORRECT_KEY | AT91C_MC_FCMD_UNLOCK | (AT91C_MC_PAGEN & (page << 8));       // unlock page command
    while ((AT91C_BASE_MC->MC1_FSR & AT91C_MC_FRDY) != AT91C_MC_FRDY);                                           // wait until the end of command
    for (i = 0; i < TFSport_NOR_PAGESIZE; i += sizeof(U32)) *(U32 *)(&page_addr[i]) = *(U32 *)(&src[i]);         // copy to the write buffer
    AT91C_BASE_MC->MC1_FCR = AT91C_MC_CORRECT_KEY | AT91C_MC_FCMD_START_PROG | (AT91C_MC_PAGEN & (page << 8));   // start programming command
    while ((AT91C_BASE_MC->MC1_FSR & AT91C_MC_FRDY) != AT91C_MC_FRDY);                                           // wait until the end of command
    OS_EXIT_CRITICAL();
    if (AT91C_BASE_MC->MC1_FSR & (AT91C_MC_PROGE | AT91C_MC_LOCKE)) return TFS_WRITE_ERR;                        // check for errors
    return TFS_NO_ERR;
}


/*
************************************************************
*                    INIT NOR-MEMORY
************************************************************
*/
__align(4) U08 TFSport_page[TFSport_NOR_PAGESIZE]; // must be 32bit aligned

U08 TFSPort_Init(void)
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif

    OS_ENTER_CRITICAL();
    AT91C_BASE_MC->MC1_FMR = ((AT91C_MC_FMCN) & (((1611*(MCU_MAIN_CLOCK >> 10)) >> 20) << 16)) | AT91C_MC_FWS_3FWS;
    AT91C_BASE_PMC->PMC_MOR = ((AT91C_CKGR_OSCOUNT & (0x06 << 8)) | AT91C_CKGR_MOSCEN);
    while(!(AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MOSCS));
    AT91C_BASE_PMC->PMC_MCKR = AT91C_PMC_CSS_MAIN_CLK;
    while (!(AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MCKRDY));
    OS_EXIT_CRITICAL();
    
    return(TFS_NO_ERR);
}

/*
************************************************************
*                  WRITE INTO NOR-MEMORY
************************************************************
*/
U08 TFSPort_Write(U08 OS_HUGE *dest, OS_CONST U08 OS_HUGE *source, TFS_LONG length)
{
    U08     *fdest, *fsrc;
    U32     segoffs, i, im;
    U08     ret;

    if((source >= (U08 OS_HUGE *)TFS_START) && (source < ((U08 OS_HUGE *)(TFS_START) + TFS_SIZE)))
        return(TFS_WR_PTR);                                // ERROR - source is in TFS-area
    if((dest < (U08 OS_HUGE *)TFS_START) || ((dest + length) > ((U08 OS_HUGE *)(TFS_START) + TFS_SIZE)))
        return(TFS_WR_PTR);                                // ERROR - destination is not in TFS-area
    segoffs =                 (U32)dest &  (TFSport_NOR_PAGESIZE - 1);
    fdest   = (U08 OS_HUGE *)((U32)dest & ~(TFSport_NOR_PAGESIZE - 1));
    while(length) {
        if ((segoffs > 0) || (length < TFSport_NOR_PAGESIZE)) {
            for (i = 0; i < TFSport_NOR_PAGESIZE; i += sizeof(U32)) *(U32 *)(&TFSport_page[i]) = *(U32 *)(&fdest[i]);
            im = ((segoffs + length) > TFSport_NOR_PAGESIZE)? TFSport_NOR_PAGESIZE : segoffs + length;
            for (i = segoffs; i < im; i++) TFSport_page[i] = *source++;
            length -= (im - segoffs);
            fsrc    = TFSport_page;
        } else {
            if (((U32)source & 0x03) != 0) {
                for (i = 0; i < TFSport_NOR_PAGESIZE; i++) TFSport_page[i] = source[i];
                fsrc    = TFSport_page;
            } else {
                fsrc    = source;
            }
            length -= TFSport_NOR_PAGESIZE;
            source += TFSport_NOR_PAGESIZE;
        }
        segoffs = 0;
        ret = TFSport_erase_page(fdest);
        if (ret != TFS_NO_ERR) return ret;
        ret = TFSport_write_page(fdest, fsrc);
        if (ret != TFS_NO_ERR) return ret;
        fdest += TFSport_NOR_PAGESIZE;
    }
    return(TFS_NO_ERR);
}

/*
************************************************************
*                      FILL NOR-MEMORY
************************************************************
*/
U08 TFSPort_Fill(U08 OS_HUGE *dest, U08 value, TFS_LONG length)
{
    U08     *fdest;
    U32     segoffs, i, im;
    U08     ret;

    if((dest < (U08 OS_HUGE *)TFS_START) || ((dest + length) > ((U08 OS_HUGE *)(TFS_START) + TFS_SIZE)))
        return(TFS_WR_PTR);                                // ERROR - destination is not in TFS-area
    segoffs =                 (U32)dest &  (TFSport_NOR_PAGESIZE - 1);
    fdest   = (U08 OS_HUGE *)((U32)dest & ~(TFSport_NOR_PAGESIZE - 1));
    while(length) {
        if ((segoffs > 0) || (length < TFSport_NOR_PAGESIZE)) {
            for (i = 0; i < TFSport_NOR_PAGESIZE; i += sizeof(U32)) *(U32 *)(&TFSport_page[i]) = *(U32 *)(&fdest[i]);
            im = (TFSport_NOR_PAGESIZE < length)? TFSport_NOR_PAGESIZE : segoffs + length;
            for (i = segoffs; i < im; i++) TFSport_page[i] = value;
            length -= (im - segoffs);
        } else {
            for (i = 0; i < TFSport_NOR_PAGESIZE; i++) TFSport_page[i] = value;
            length -= TFSport_NOR_PAGESIZE;
        }
        segoffs = 0;
        ret = TFSport_erase_page(fdest);
        if (ret != TFS_NO_ERR) return ret;
        ret = TFSport_write_page(fdest, TFSport_page);
        if (ret != TFS_NO_ERR) return ret;
        fdest += TFSport_NOR_PAGESIZE;
    }
    return(TFS_NO_ERR);
}

