/***********************************************************
*                       pC/TFS 2.38b                       *
*                 a FREE - Tiny File System                *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                      TFSHost_LINUX.c                     *
*                      LINUX Host Port                     *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/
#include "../../inc/pC_TFS.h"
#include "../../inc/TFS_string.h"
#include "../../inc/LINUX/TFSHost_LINUX.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>

/****************** Globals im RAM ************************/

U08 OS_HUGE  *org_START;
S32           difference;

#ifndef TFS_START                                          // if no (compiler unknown) TFS-area defined ...
 U08 TFS_START[TFS_SIZE];                                  // ... create an array
#endif

extern TFS_DIRFILE OS_HUGE ROOT;
extern TFS_DIRFILE OS_HUGE *ROOT_PTR; 

/*
************************************************************
*                    INIT MEMORY
************************************************************
*/
U08 TFSPort_Init(void)
{
    return(TFS_NO_ERR);                                    // nothing to init here
}

/*
************************************************************
*                    WRITE INTO RAM
************************************************************
*/
U08 TFSPort_Write(U08 OS_HUGE *dest, OS_CONST U08 OS_HUGE *source, TFS_LONG length) KC51_REENTRANT
{
    TFS_LONG byte;

    if((source >= (U08 OS_HUGE *)TFS_START) && (source < ((U08 OS_HUGE *)(TFS_START) + TFS_SIZE)))
        return(TFS_WR_PTR);                                // ERROR - source is in TFS-area
    if((dest < (U08 OS_HUGE *)TFS_START) || ((dest + length) > ((U08 OS_HUGE *)(TFS_START) + TFS_SIZE)))
        return(TFS_WR_PTR);                                // ERROR - destination is not in TFS-area
    for(byte = 0; byte < length; byte++)
        dest[byte] = source[byte];                         // write bytes
    return(TFS_NO_ERR);
}

/*
************************************************************
*                        FILL RAM
************************************************************
*/
U08 TFSPort_Fill(U08 OS_HUGE *dest, U08 value, TFS_LONG length) KC51_REENTRANT
{
    TFS_LONG byte;

    if((dest < (U08 OS_HUGE *)TFS_START) || ((dest + length) > ((U08 OS_HUGE *)(TFS_START) + TFS_SIZE)))
        return(TFS_WR_PTR);                                // ERROR - destination is not in TFS-area
    for(byte = 0; byte < length; byte++)
        dest[byte] = value;                                // fill with value
    return(TFS_NO_ERR);
}

/*
************************************************************
*       RELOCATE all Entries of a Level in new Adress-Space 
*                          (recursiv)
************************************************************
*/
S16 TFSHost_RelocateLevel(TFS_DIRFILE OS_HUGE *EntryPTR)
{
    S16 returnOk=TFS_NO_ERR;

    do {
        if(EntryPTR->Entry.Attr & (TFS_ATTR_FILE | TFS_ATTR_DIR) && EntryPTR->Sub) {
            if(EntryPTR->Entry.Attr & TFS_ATTR_LINK)
                EntryPTR->Sub=(TFS_ENTRY OS_HUGE *)((U08 OS_HUGE *)(EntryPTR->Sub) - difference);
            else {
                EntryPTR->Sub=(TFS_ENTRY OS_HUGE *)((U08 OS_HUGE *)(EntryPTR->Sub) - difference);
                returnOk=TFSHost_RelocateLevel((TFS_DIRFILE OS_HUGE *)(EntryPTR->Sub));
            }
        }
        EntryPTR->Entry.PrevLevel=(TFS_ENTRY OS_HUGE *)((U08 OS_HUGE *)(EntryPTR->Entry.PrevLevel) - difference);
#if (TFSDBLLINKED==1)
        if(EntryPTR->Entry.Prev)
            EntryPTR->Entry.Prev=(TFS_ENTRY OS_HUGE *)((U08 OS_HUGE *)(EntryPTR->Entry.Prev) - difference);
#endif
        if(EntryPTR->Entry.Next)
            EntryPTR->Entry.Next=(TFS_ENTRY OS_HUGE *)((U08 OS_HUGE *)(EntryPTR->Entry.Next) - difference);
        EntryPTR=(TFS_DIRFILE OS_HUGE *)(EntryPTR->Entry.Next);
    } while(EntryPTR && returnOk==TFS_NO_ERR);
    return(TFS_NO_ERR);
}

/*
************************************************************
*              RELOCATE DRIVE in new Adress-Space
************************************************************
*/
S16 TFSHost_Relocate()
{
    S16                  returnOk=TFS_NO_ERR;
    TFS_MEM   OS_HUGE   *cur_TFSM;
    TFS_MEM   OS_HUGE   *tmp_TFSM;

    cur_TFSM=(TFS_MEM OS_HUGE *)&TFS_START;

    returnOk=TFS_NO_ERR;
    if(TFSstrcmp(ROOT_PTR->Name, ROOT.Name)
#if (TFSDBLLINKED==1)
    || ROOT_PTR->Entry.Prev != 0
#endif
    || ROOT_PTR->Entry.Next != 0 || ROOT_PTR->Entry.PrevLevel != 0)
        return(TFS_INVALID);

    difference=(S32)((U08 OS_HUGE *)(org_START) - (U08 OS_HUGE *)(&TFS_START));
    if(difference) {
        do {
            tmp_TFSM=(TFS_MEM OS_HUGE *)((U08 OS_HUGE *)(cur_TFSM->Next) - difference);
#if (TFSDBLLINKED==1)
            if((TFS_MEM OS_HUGE *)((U08 OS_HUGE *)(tmp_TFSM->Prev) - difference) != cur_TFSM)   
                return(TFS_INVALID);
            tmp_TFSM->Prev=(TFS_MEM OS_HUGE *)((U08 OS_HUGE *)(tmp_TFSM->Prev) - difference);
#endif
            cur_TFSM->Next=(TFS_MEM OS_HUGE *)((U08 OS_HUGE *)(cur_TFSM->Next) - difference);
            cur_TFSM=cur_TFSM->Next;
        } while(cur_TFSM->Next);
        if(ROOT_PTR->Sub) {
            ROOT_PTR->Sub=(TFS_ENTRY OS_HUGE *)((U08 OS_HUGE *)(ROOT_PTR->Sub) - difference);
            returnOk=TFSHost_RelocateLevel((TFS_DIRFILE OS_HUGE *)(ROOT_PTR->Sub));
        }
    }
    return(returnOk);
}

/*
************************************************************
*                LOAD DRIVE from a real File
************************************************************
*/
S16 TFS_Load()
{
    S16          returnOk=TFS_NO_ERR;
    static FILE *ifile;
    S32          fsize;
    size_t       bytesread=0;

    ifile=fopen(IMAGE_FILE,"rb");
    if(ifile == NULL)
        return 0;
    fseek(ifile, 0L, SEEK_END);
    fsize=ftell(ifile);
    if(fsize != TFS_SIZE + sizeof(U08 OS_HUGE *))
        fclose(ifile);
    else {
        rewind(ifile);			
        bytesread=fread( &org_START, 1,  sizeof(U08 OS_HUGE *), ifile);
        bytesread=fread( &TFS_START, 1, TFS_SIZE, ifile);
        fclose(ifile);
        returnOk=TFSHost_Relocate();
    }
    return(returnOk);
}

/*
************************************************************
*                  FLUSH DRIVE to a real File
************************************************************
*/
S16 TFS_Flush()
{
    static FILE *ofile;
    size_t       byteswritten=0;
    void        *pTFS_START=&TFS_START;

    ofile=fopen(IMAGE_FILE,"wb+");
    rewind(ofile);			
    byteswritten=fwrite( &pTFS_START, 1, sizeof(U08 OS_HUGE *), ofile);
    byteswritten=fwrite( &TFS_START, 1, TFS_SIZE, ofile);
    fclose (ofile);
    return(TFS_NO_ERR);
}

/**********************************************************/




