/***********************************************************
*                       pC/TFS 2.38b                       *
*                 a FREE - Tiny File System                *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                  TFS_Port_RAM_win32.c                    *
*              RAM Port Windows32 extension                *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/
#include "iostream"
#include <windows.h>
#include <tchar.h>
#include <stdio.h>
#include <strsafe.h>

extern "C" {

#include "../../inc/RAM/TFS_Port_RAM.h"
#include "../../inc/TFS_string.h"

#define IMAGE_FILE TEXT("./TFS_IMAGE.bin")

extern U08                  TFS_START[TFS_SIZE];
extern TFS_DIRFILE OS_HUGE  ROOT;
extern TFS_DIRFILE OS_HUGE *ROOT_PTR; 

U32           org_START;
S32           difference;

/*
************************************************************
*       RELOCATE all Entries of a Level in new Adress-Space 
*                          (recursiv)
************************************************************
*/
S16 TFSHost_RelocateLevel(TFS_DIRFILE OS_HUGE *EntryPTR)
{
    S16 returnOk=TFS_NO_ERR;

    do {
        if(EntryPTR->Entry.Attr & (TFS_ATTR_FILE | TFS_ATTR_DIR) && EntryPTR->Sub) {
            if(EntryPTR->Entry.Attr & TFS_ATTR_LINK)
                EntryPTR->Sub = (TFS_ENTRY OS_HUGE *)((U08 OS_HUGE *)(EntryPTR->Sub) - difference);
            else {
                EntryPTR->Sub = (TFS_ENTRY OS_HUGE *)((U08 OS_HUGE *)(EntryPTR->Sub) - difference);
                returnOk = TFSHost_RelocateLevel((TFS_DIRFILE OS_HUGE *)(EntryPTR->Sub));
            }
        }
        EntryPTR->Entry.PrevLevel = (TFS_ENTRY OS_HUGE *)((U08 OS_HUGE *)(EntryPTR->Entry.PrevLevel) - difference);
#if (TFSDBLLINKED==1)
        if(EntryPTR->Entry.Prev)
            EntryPTR->Entry.Prev = (TFS_ENTRY OS_HUGE *)((U08 OS_HUGE *)(EntryPTR->Entry.Prev) - difference);
#endif
        if(EntryPTR->Entry.Next)
            EntryPTR->Entry.Next = (TFS_ENTRY OS_HUGE *)((U08 OS_HUGE *)(EntryPTR->Entry.Next) - difference);
        EntryPTR=(TFS_DIRFILE OS_HUGE *)(EntryPTR->Entry.Next);
    } while((EntryPTR) && (returnOk == TFS_NO_ERR));
    return(TFS_NO_ERR);
}

/*
************************************************************
*              RELOCATE DRIVE in new Adress-Space
************************************************************
*/
S16 TFSHost_Relocate()
{
    S16                  returnOk=TFS_NO_ERR;
    TFS_MEM   OS_HUGE   *cur_TFSM;
    TFS_MEM   OS_HUGE   *tmp_TFSM;

    cur_TFSM = (TFS_MEM OS_HUGE *)&TFS_START;

    returnOk = TFS_NO_ERR;
    if(TFSstrcmp(ROOT_PTR->Name, ROOT.Name)
#if (TFSDBLLINKED==1)
    || ROOT_PTR->Entry.Prev != 0
#endif
    || ROOT_PTR->Entry.Next != 0 || ROOT_PTR->Entry.PrevLevel != 0)
        return(TFS_INVALID);
    difference = (S32)((U08 OS_HUGE *)(org_START) - (U08 OS_HUGE *)(&TFS_START));
    if(difference) {
        do {
            tmp_TFSM = (TFS_MEM OS_HUGE *)((U08 OS_HUGE *)(cur_TFSM->Next) - difference);
#if (TFSDBLLINKED==1)
            if((TFS_MEM OS_HUGE *)((U08 OS_HUGE *)(tmp_TFSM->Prev) - difference) != cur_TFSM)
                return(TFS_INVALID);
            tmp_TFSM->Prev = (TFS_MEM OS_HUGE *)((U08 OS_HUGE *)(tmp_TFSM->Prev) - difference);
#endif
            cur_TFSM->Next = (TFS_MEM OS_HUGE *)((U08 OS_HUGE *)(cur_TFSM->Next) - difference);
            cur_TFSM = cur_TFSM->Next;
        } while(cur_TFSM->Next);
        if(ROOT_PTR->Sub) {
            ROOT_PTR->Sub = (TFS_ENTRY OS_HUGE *)((U08 OS_HUGE *)(ROOT_PTR->Sub) - difference);
            returnOk = TFSHost_RelocateLevel((TFS_DIRFILE OS_HUGE *)(ROOT_PTR->Sub));
        }
    }
    return(returnOk);
}

/*
************************************************************
*                LOAD DRIVE from a real File
************************************************************
*/
S16 TFS_Load()
{
    S16           returnOk = TFS_NO_ERR;
    HANDLE        ifile;
    S32           fsize;
    DWORD         bytesread = 0;

    ifile = CreateFile(IMAGE_FILE, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if(ifile == INVALID_HANDLE_VALUE)
        return 0;
    SetFilePointer(ifile, 0, NULL, FILE_BEGIN);
    fsize = GetFileSize(ifile, NULL);
    if(fsize != (TFS_SIZE + sizeof(U32)))
        CloseHandle(ifile);
    else {
        SetFilePointer(ifile, 0, NULL, FILE_BEGIN);
        ReadFile( ifile, &org_START, sizeof(U32), &bytesread, NULL);
        ReadFile( ifile, TFS_START, TFS_SIZE, &bytesread, NULL);
        CloseHandle(ifile);
        returnOk = TFSHost_Relocate();
    }
    return(returnOk);
}

/*
************************************************************
*                  FLUSH DRIVE to a real File
************************************************************
*/
S16 TFS_Flush()
{
    HANDLE       ofile;
    DWORD        byteswritten = 0;
    U32          pTFS_START = (U32)&TFS_START;

    ofile = CreateFile(IMAGE_FILE, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    SetFilePointer(ofile, 0, NULL, FILE_BEGIN);
    WriteFile( ofile, &pTFS_START, sizeof(U32), &byteswritten, NULL);
    WriteFile( ofile, TFS_START, TFS_SIZE, &byteswritten, NULL);
    CloseHandle(ofile);
    return(TFS_NO_ERR);
}

/**********************************************************/
}     // extern "C"

